@@ -9,17 +9,18 @@ import tempfile
 
 import portage
 from portage import shutil, os
-from portage.const import BASH_BINARY
+from portage.const import BASH_BINARY, PORTAGE_PYM_PATH
 from portage.tests import TestCase
 from portage.tests.resolver.ResolverPlayground import ResolverPlayground
 from portage.tests.util.test_socks5 import AsyncHTTPServer
 from portage.util.configparser import ConfigParserError
+from portage.util.futures import asyncio
 from portage.util.futures.executor.fork import ForkExecutor
 from portage.util._async.SchedulerInterface import SchedulerInterface
 from portage.util._eventloop.global_event_loop import global_event_loop
 from portage.package.ebuild.config import config
 from portage.package.ebuild.digestgen import digestgen
-from portage.package.ebuild.fetch import (_download_suffix, FlatLayout,
+from portage.package.ebuild.fetch import (_download_suffix, fetch, FlatLayout,
 		FilenameHashLayout, MirrorLayoutConfig)
 from _emerge.EbuildFetcher import EbuildFetcher
 from _emerge.Package import Package
@@ -29,6 +30,13 @@ class EbuildFetchTestCase(TestCase):
 
 	def testEbuildFetch(self):
 
+		user_config = {
+			"make.conf":
+				(
+					'GENTOO_MIRRORS="{scheme}://{host}:{port}"',
+				),
+		}
+
 		distfiles = {
 			'bar': b'bar\n',
 			'foo': b'foo\n',
@@ -37,17 +45,28 @@ class EbuildFetchTestCase(TestCase):
 		ebuilds = {
 			'dev-libs/A-1': {
 				'EAPI': '7',
-				'RESTRICT': 'primaryuri',
 				'SRC_URI': '''{scheme}://{host}:{port}/distfiles/bar.txt -> bar
 					{scheme}://{host}:{port}/distfiles/foo.txt -> foo''',
 			},
 		}
 
 		loop = SchedulerInterface(global_event_loop())
+
+		def run_async(func, *args, **kwargs):
+			with ForkExecutor(loop=loop) as executor:
+				return loop.run_until_complete(loop.run_in_executor(executor,
+					functools.partial(func, *args, **kwargs)))
+
 		scheme = 'http'
 		host = '127.0.0.1'
 		content = {}
+
+		content['/distfiles/layout.conf'] = b'[structure]\n0=flat\n'
+
 		for k, v in distfiles.items():
+			# mirror path
+			content['/distfiles/{}'.format(k)] = v
+			# upstream path
 			content['/distfiles/{}.txt'.format(k)] = v
 
 		with AsyncHTTPServer(host, content, loop) as server:
@@ -58,7 +77,16 @@ class EbuildFetchTestCase(TestCase):
 					scheme=scheme, host=host, port=server.server_port)
 				ebuilds_subst[cpv] = metadata
 
-			playground = ResolverPlayground(ebuilds=ebuilds_subst, distfiles=distfiles)
+			user_config_subst = user_config.copy()
+			for configname, configdata in user_config.items():
+
+				configdata_sub = []
+				for line in configdata:
+					configdata_sub.append(line.format(
+					scheme=scheme, host=host, port=server.server_port))
+				user_config_subst[configname] = tuple(configdata_sub)
+
+			playground = ResolverPlayground(ebuilds=ebuilds_subst, distfiles=distfiles, user_config=user_config_subst)
 			ro_distdir = tempfile.mkdtemp()
 			eubin = os.path.join(playground.eprefix, "usr", "bin")
 			try:
@@ -77,6 +105,84 @@ class EbuildFetchTestCase(TestCase):
 				portdb = root_config.trees["porttree"].dbapi
 				settings = config(clone=playground.settings)
 
+				# Demonstrate that fetch preserves a stale file in DISTDIR when no digests are given.
+				foo_uri = {'foo': ('{scheme}://{host}:{port}/distfiles/foo'.format(scheme=scheme, host=host, port=server.server_port),)}
+				foo_path = os.path.join(settings['DISTDIR'], 'foo')
+				foo_stale_content = b'stale content\n'
+				with open(foo_path, 'wb') as f:
+					f.write(b'stale content\n')
+
+				self.assertTrue(bool(run_async(fetch, foo_uri, settings, try_mirrors=False)))
+
+				with open(foo_path, 'rb') as f:
+					self.assertEqual(f.read(), foo_stale_content)
+				with open(foo_path, 'rb') as f:
+					self.assertNotEqual(f.read(), distfiles['foo'])
+
+				# Use force=True to update the stale file.
+				self.assertTrue(bool(run_async(fetch, foo_uri, settings, try_mirrors=False, force=True)))
+
+				with open(foo_path, 'rb') as f:
+					self.assertEqual(f.read(), distfiles['foo'])
+
+				# Test force=True with FEATURES=skiprocheck, using read-only DISTDIR.
+				# FETCHCOMMAND is set to temporarily chmod +w DISTDIR. Note that
+				# FETCHCOMMAND must perform atomic rename itself due to read-only
+				# DISTDIR.
+				with open(foo_path, 'wb') as f:
+					f.write(b'stale content\n')
+				orig_fetchcommand = settings['FETCHCOMMAND']
+				orig_distdir_mode = os.stat(settings['DISTDIR']).st_mode
+				temp_fetchcommand = os.path.join(eubin, 'fetchcommand')
+				with open(temp_fetchcommand, 'w') as f:
+					f.write("""
+					set -e
+					URI=$1
+					DISTDIR=$2
+					FILE=$3
+					trap 'chmod a-w "${DISTDIR}"' EXIT
+					chmod ug+w "${DISTDIR}"
+					%s
+					mv -f "${DISTDIR}/${FILE}.__download__" "${DISTDIR}/${FILE}"
+				""" % orig_fetchcommand.replace('${FILE}', '${FILE}.__download__'))
+				settings['FETCHCOMMAND'] = '"%s" "%s" "${URI}" "${DISTDIR}" "${FILE}"' % (BASH_BINARY, temp_fetchcommand)
+				settings.features.add('skiprocheck')
+				settings.features.remove('distlocks')
+				os.chmod(settings['DISTDIR'], 0o555)
+				try:
+					self.assertTrue(bool(run_async(fetch, foo_uri, settings, try_mirrors=False, force=True)))
+				finally:
+					settings['FETCHCOMMAND'] = orig_fetchcommand
+					os.chmod(settings['DISTDIR'], orig_distdir_mode)
+					settings.features.remove('skiprocheck')
+					settings.features.add('distlocks')
+					os.unlink(temp_fetchcommand)
+
+				with open(foo_path, 'rb') as f:
+					self.assertEqual(f.read(), distfiles['foo'])
+
+				# Test emirrordist invocation.
+				emirrordist_cmd = (portage._python_interpreter, '-b', '-Wd',
+					os.path.join(self.bindir, 'emirrordist'),
+					'--distfiles', settings['DISTDIR'],
+					'--config-root', settings['EPREFIX'],
+					'--repositories-configuration', settings.repositories.config_string(),
+					'--repo', 'test_repo', '--mirror')
+
+				env = os.environ.copy()
+				env['PYTHONPATH'] = ':'.join(
+					filter(None, [PORTAGE_PYM_PATH] + os.environ.get('PYTHONPATH', '').split(':')))
+
+				for k in distfiles:
+					os.unlink(os.path.join(settings['DISTDIR'], k))
+
+				proc = loop.run_until_complete(asyncio.create_subprocess_exec(*emirrordist_cmd, env=env))
+				self.assertEqual(loop.run_until_complete(proc.wait()), 0)
+
+				for k in distfiles:
+					with open(os.path.join(settings['DISTDIR'], k), 'rb') as f:
+						self.assertEqual(f.read(), distfiles[k])
+
 				# Tests only work with one ebuild at a time, so the config
 				# pool only needs a single config instance.
 				class config_pool:
@@ -342,3 +448,29 @@ class EbuildFetchTestCase(TestCase):
 '''
 		self.assertRaises(ConfigParserError, mlc.read_from_file,
 				io.StringIO(conf))
+
+	def test_filename_hash_layout_get_filenames(self):
+		layouts = (
+			FlatLayout(),
+			FilenameHashLayout('SHA1', '4'),
+			FilenameHashLayout('SHA1', '8'),
+			FilenameHashLayout('SHA1', '8:16'),
+			FilenameHashLayout('SHA1', '8:16:24'),
+		)
+		filename = 'foo-1.tar.gz'
+
+		for layout in layouts:
+			distdir = tempfile.mkdtemp()
+			try:
+				path = os.path.join(distdir, layout.get_path(filename))
+				try:
+					os.makedirs(os.path.dirname(path))
+				except OSError:
+					pass
+
+				with open(path, 'wb') as f:
+					pass
+
+				self.assertEqual([filename], list(layout.get_filenames(distdir)))
+			finally:
+				shutil.rmtree(distdir)
